#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_UseX64=n
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#include <file.au3>

;setup variables
$ClientIP = "" ;modbus tcp gateway IP
$SlaveID = "" ;modbus slave id
$ClientDesc = "" ;description given to specific slave
$GoodPoll = 0 ;used to see if a good poll was made
$PollCount = 0 ;used to count how many polls have been made.  Total of 3 polls made before the slave is given up as dead.
$BCM = 0 ; just used to look for the first BCM
$WriteString = "" ; The string written to file

;make sure the year folder exists
DirCreate(@ScriptDir & "\" & @YEAR)

;read in our config file
Dim $aConfigFile
If Not _FileReadToArray(@ScriptDir & "\config.txt",$aConfigFile) Then
   MsgBox(4096,"Error", " Error reading log to Array     error:" & @error)
   Exit
EndIf

;loop through all of our BCMs to poll
For $x = 1 to $aConfigFile[0]
	if StringInStr("  " & $aConfigFile[$x], "#") <= 0 Then
		;not a comment, process it.
		;set the variables via the config file
		$ClientIP = StringLeft($aConfigFile[$x], StringInStr($aConfigFile[$x],",") - 1)
		$SlaveID = StringMid($aConfigFile[$x], StringInStr($aConfigFile[$x],",") + 1,StringInStr($aConfigFile[$x],",",0,2) - StringInStr($aConfigFile[$x],",") - 1)
		$ClientDesc = StringMid($aConfigFile[$x],StringInStr($aConfigFile[$x],",",0,2) + 1)
		$PollCount = 0 ;set our poll counter to zero
		PollClient() ;poll our BCM
	EndIf
Next

Func PollClient()
	;zero out variables
	$GoodPoll = 0
	$PollCount = 0
	$BCM = 0
	$WriteString = ""

	;the modpoll program is fickle about output redirection.  What we have to do is
	;create a batch file, then pipe the output of the batch to file.
	FileDelete(@ScriptDir & "\process.bat") ;delete the old batch file.

	;write the new batchfile with our current BCM client info
	$batfile = FileOpen(@ScriptDir & "\process.bat", 1)

	; Check if file opened for writing OK
	If $batfile = -1 Then
		MsgBox(0, "Error", "Unable to open file.",2)
		Exit
	EndIf

	FileWriteLine($batfile, '"' & @ScriptDir & '\modpoll.exe" -m tcp -c 42 -r 1 -1 -a ' & $SlaveID & " " & $ClientIP)
	FileClose($batfile)

	Sleep(500) ;Slight pause after file creation

	;Attempt to poll a total of 3 times
	while $PollCount < 3 and $GoodPoll == 0
		ToolTip($ClientDesc & "~" & $PollCount & "~" & $GoodPoll,0,0)
		PullValues()
	WEnd

	;if the polling was successful.
	if $GoodPoll == 1 Then
		;write this line to file
		$timeStamp = @year & "/" & @MON & "/" & @MDAY & " " & @HOUR & ":" & @MIN & ":" & @SEC
		$InfoFile = FileOpen(@ScriptDir & "\" & @YEAR & "\" & @MON & "-" & $ClientDesc & ".csv", 1)

		; Check if file opened for writing OK
		If $InfoFile = -1 Then
			MsgBox(0, "Error", "Unable to open file.")
			Exit
		EndIf

		FileWriteLine($InfoFile, $timeStamp & "," & $WriteString & @CRLF)

		FileClose($InfoFile)

	EndIf
EndFunc

Func PullValues()
	$PollCount = $PollCount + 1
	FileDelete(@ScriptDir & "\temp.txt") ;Delete the old temp working file.
	;run the batch file and redirect it to temp.txt
	RunWait(@ComSpec & " /c " & FileGetShortName(@ScriptDir & "\process.bat") & ' > ' & FileGetShortName(@ScriptDir) & "\temp.txt", "", @SW_HIDE)

	Dim $aTempFile
	If Not _FileReadToArray(@ScriptDir & "\temp.txt",$aTempFile) Then
	MsgBox(4096,"Error", " Error reading log to Array     error:" & @error)
	Exit
	EndIf
	For $y = 1 to $aTempFile[0]
		if $GoodPoll == 1 Then
			;we've already found our poll info, so start pulling BCMs
			$BCM = $BCM + 1
			if $BCM == 1 Then
				;we found our first bcm, so process accordingly
				$WriteString = StringMid($aTempFile[$y],StringInStr($aTempFile[$y],':') + 2)
			Else
				;process normally
				$WriteString = $WriteString & "," & StringMid($aTempFile[$y],StringInStr($aTempFile[$y],':') + 2)
			EndIf
		EndIf
		if StringInStr("   " & $aTempFile[$y],"Polling slave ...") > 0 Then
			;we found a good one.  This is the check for the beginning of our BCM info.
			$GoodPoll = 1
		EndIf

	Next

	if $GoodPoll == 0 Then
		;if it failed to properly poll, sleep for 1 second and try again.
		sleep(1000)
	EndIf
EndFunc